#include <driver/usb.h>
#include <btron/proctask.h>
#include <bstdio.h>

LOCAL	W	readReg(W addr, UB reg)
{
	ERR	err;
	UB	d;
	usbDeviceRequest	r;

	r.bmRequestType = 0xc0;
	r.bRequest = 0x00;
	r.wValue = 0x0000;
	r.wIndex = reg;
	r.wLength = sizeof(d);

	err = usbRequestDevice(addr, &r, &d, r.wLength, NULL);
	if (err < ER_OK) return -1;
	return d;
}

LOCAL	W	readRegH(W addr, UB reg)
{
	ERR	err;
	UH	d;
	usbDeviceRequest	r;

	r.bmRequestType = 0xc0;
	r.bRequest = 0x00;
	r.wValue = 0x0000;
	r.wIndex = reg;
	r.wLength = sizeof(d);

	err = usbRequestDevice(addr, &r, (UB *)&d, r.wLength, NULL);
	if (err < ER_OK) return -1;
	return d;
}

LOCAL	W	writeReg(W addr, UB reg, UB dat)
{
	ERR	err;
	usbDeviceRequest	r;

	r.bmRequestType = 0x40;
	r.bRequest = 0x03;
	r.wValue = dat;
	r.wIndex = reg;
	r.wLength = 0;

	err = usbRequestDevice(addr, &r, NULL, 0, NULL);
	if (err < ER_OK) printf("error %d\n", err);
	return err;
}

#define	NCR	0x00
#define	EPCR	0x0b
#define	EPAR	0x0c
#define	EPDR	0x0d
#define	PRR	0x1f

LOCAL	W	readX(W addr, W phy, UB reg)
{
	if (phy < 0) {
		writeReg(addr, EPAR, reg);
		writeReg(addr, EPCR, 0x04);
	} else {
		writeReg(addr, EPAR, ((phy & 0x03) * 0x40) | (reg & 0x3f));
		writeReg(addr, EPCR, 0x0c);
	}

	wai_prc(10);
	if (readReg(addr, EPCR) & 1) printf("!");

	writeReg(addr, EPCR, 0);
	return readRegH(addr, EPDR);
}

LOCAL	void	test(W did)
{
	W	i, j;

	printf("eprom\n");
	for (i = 0; i < 0x80; i++) {
		printf("%04x ", readX(did, -1, i));
		if (!((i + 1) % 16)) printf("\n");
	}
	printf("\n");

	printf("phy\n");
	for (i = 0; i < 4; i++) {
		for (j = 0; j < 0x20; j++) {
			printf("%04x ", readX(did, i, j));
			if (!((j + 1) % 16)) printf("\n");
		}
		printf("\n");
	}

	printf("reg\n");
	for (i = 0; i < 0x40; i++) {
		printf("%02x ", readReg(did, i));
		if (!((i + 1) % 16)) printf("\n");
	}
	printf("\n");
}

LOCAL	void	reset(W did, W val)
{
	writeReg(did, NCR, val);
	wai_prc(20);

	writeReg(did, PRR, 1);
	wai_prc(20);

	writeReg(did, PRR, 0);
	wai_prc(20);
}

EXPORT	ERR	main(W argc, TC *argv[])
{
	ERR	err;
	W	did;
	usbDeviceDescriptor	d;

	for (did = 0; did < 32; did++) {
		err = usbDescriptorDevice(did, (UB *)&d, sizeof(d), NULL);
		if (err < ER_OK) continue;
		if (d.idVendor == 0x0fe6 &&
		    d.idProduct == 0x9700) break;
	}
	if (did == 32) {
		printf("no device\n");
		goto fin0;
	}

	err = usbOpenDevice(did);
	if (err < ER_OK) {
		printf("usbOpenDevice %d\n", err);
		goto fin0;
	}

	err = usbConfigDevice(did, 1);
	if (err < ER_OK) {
		printf("usbConfigDevice %d\n", err);
		goto fin1;
	}

	reset(did, 0x01);
	test(did);

	reset(did, 0x81);
	test(did);

fin1:
	usbCloseDevice(did);
fin0:
	return 0;
}
